/*
 * Copyright (c) 2000, 2001 Murray Stokely
 * Copyright (c) 2000, 2001 BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Driver for National Instruments PCI E-Series data acquisition cards.
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, January 1999 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *
 */

#include <sys/param.h>  /* defines used in kernel.h */
#include <sys/module.h>
#include <sys/systm.h>  /* uprintf */ 
#include <sys/errno.h>
#include <sys/kernel.h> /* types used in module initialization */
#include <sys/conf.h>   /* cdevsw struct */
#include <sys/uio.h>    /* uio struct */
#include <sys/malloc.h>
#include <sys/fcntl.h>		/* So we can get open modes */
#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/bus.h>	/* structs, prototypes for pci bus stuff */
#include <sys/rman.h>
#include <sys/types.h>
#include <sys/pciio.h>  /* For IOCTL _IOWR macros */
#include <sys/mutex.h>

#include "niio.h"
#include "ni.h"
#include "nireg.h"


int
dio_ioctl(struct ni_softc *sc, u_long cmd, caddr_t data, int fflag, struct proc *p) 
{
    int *arg = (int *) data;

    switch (cmd) {
    case NICTL_DIO_CONFPORT:
	uprintf("DIO Confport! %d\n", *arg);
	return 0;
    case NICTL_DIO_SELPORT:
	uprintf("DIO Selport! %d\n", *arg);
	return 0;
    case NICTL_DIO_WRMASK:
	sc->wrmask = *arg;
	DAQ_STC_Windowed_Mode_Write(sc, NI_DIO_CONTROL, *arg);
	return 0;
    }
    return -EINVAL;
}
