/*
 * Copyright (c) 2001 Murray Stokely
 * Copyright (c) 2001 BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Driver for National Instruments PCI E-Series data aquisition cards.
 * General Purpose Counter/Timer Functions
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, January 1999 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *
 */

#include <sys/param.h>  /* defines used in kernel.h */
#include <sys/module.h>
#include <sys/systm.h>  /* uprintf */ 
#include <sys/errno.h>
#include <sys/kernel.h> /* types used in module initialization */
#include <sys/conf.h>   /* cdevsw struct */
#include <sys/uio.h>    /* uio struct */
#include <sys/malloc.h>
#include <sys/fcntl.h>		/* So we can get open modes */
#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/bus.h>	/* structs, prototypes for pci bus stuff */
#include <sys/rman.h>
#include <sys/types.h>
#include <sys/pciio.h>  /* For IOCTL _IOWR macros */
#include <sys/mutex.h>

#include "niio.h"
#include "ni.h"
#include "nireg.h"

int
gpct0_ioctl(struct ni_softc *sc, u_long cmd, caddr_t data, int fflag, struct proc *p) 
{
    int *arg = (int *) data;

    if (arg)
	printf("yep, there's an arg!\n");

    switch (cmd) {
    case NICTL_GPCT_RESET:
	uprintf("GPCT0 Reset!\n");
	G0_Reset_All(sc);
	return 0;
    case NICTL_GPCT_ARM:
	uprintf("GPCT0 Arm!\n");
	G0_Arm_All(sc);
	return 0;
    case NICTL_GPCT_SETUP:
	uprintf("GPCT0 Setup!\n");
/*	G0_Simple_Event_Counting(sc);*/
	G0_Single_Pulse_Generation(sc);
/*Buffered_Event_Counting(sc);*/
	return 0;
    }
    return -EINVAL;
}


/* [1] 4.6.1.3 */

void
G0_Reset_All(struct ni_softc *sc) {
    DAQ_STC_Windowed_Mode_Write(sc, NI_JOINT_RESET, NIB_G0_RESET);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_MODE, 0);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_COMMAND, 0);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_INPUT_SELECT, 0);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_AUTOINCREMENT, 0);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_INTERRUPT_A_ENABLE, 
				       NIB_G0_TC_INTERRUPT_ENABLE |
				       NIB_G0_GATE_INTERRUPT_ENABLE,
				       NIB_G0_TC_INTERRUPT_ENABLE |
				       NIB_G0_GATE_INTERRUPT_ENABLE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_COMMAND,
				       NIB_G0_SYNCHRONIZED_GATE,
				       NIB_G0_SYNCHRONIZED_GATE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_INTERRUPT_A_ACK,
				       NIB_G0_GATE_ERROR_CONFIRM |
				       NIB_G0_TC_ERROR_CONFIRM |
				       NIB_G0_GATE_INTERRUPT_ACK |
				       NIB_G0_TC_INTERRUPT_ACK,
				       NIB_G0_GATE_ERROR_CONFIRM |
				       NIB_G0_TC_ERROR_CONFIRM |
				       NIB_G0_GATE_INTERRUPT_ACK |
				       NIB_G0_TC_INTERRUPT_ACK);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_AUTOINCREMENT, 0);
}

void
G0_Arm_All(struct ni_softc *sc) {
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_COMMAND, NIB_G0_ARM);
}

void
G0_Simple_Event_Counting(struct ni_softc *sc) {
    uprintf("SEC!\n");
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_MODE, 0, NIB_G0_LOAD_SOURCE_SELECT);
    /* Here we setup the initial counter value -- ioctl? */
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_LOAD_A1, 0);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_LOAD_A2, 0);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_COMMAND, NIB_G0_LOAD);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_INPUT_SELECT, 
				       0 << 2| /* Source select */
				       NIB_G0_SOURCE_POLARITY |
				       1 << 7 | /* Gate select */
				       0 | /* G0_OR_GATE = 0 */
				       NIB_G0_OUTPUT_POLARITY |
/*				       NIB_G0_GATE_SELECT_LOAD_SOURCE | */
				       0, /* GATE_SELECT_LOAD_SOURCE = 0 */
				       NIB_G0_SOURCE_SELECT |
				       NIB_G0_SOURCE_POLARITY |
				       NIB_G0_GATE_SELECT |
				       NIB_G0_OR_GATE | 
				       NIB_G0_OUTPUT_POLARITY |
				       NIB_G0_GATE_SELECT_LOAD_SOURCE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_MODE,
				       0 << 13 | /* G0_GATE_POLARITY */
				       1 << 8 | /* GO_OUTPUT_MODE */
				       0 << 14 | /* G0_LOADING_ON_GATE */
				       0 << 12 | /* G0_LOADING_ON_TC */
				       1 << 0 | /* G0_GATING_MODE */
				       0 << 2 | /* G0_GATE_ON_BOTH_EDGES */
				       2 << 3 | /* G0_TRIGGER_MODE_FOR_EDGE_GATE */
				       0 << 5 | /* G0_STOP_MODE */
				       0 << 10, /* G0_COUNTING_ONCE */
				       NIB_G0_GATE_POLARITY |
				       NIB_G0_OUTPUT_MODE | 
				       NIB_G0_LOADING_ON_GATE |
				       NIB_G0_LOADING_ON_TC |
				       NIB_G0_GATING_MODE |
				       NIB_G0_GATE_ON_BOTH_EDGES |
				       NIB_G0_TRIGGER_MODE_FOR_EDGE_GATE |
				       NIB_G0_STOP_MODE |
				       NIB_G0_COUNTING_ONCE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_COMMAND,
				       1 << 5 | /* count up! */
				       0 | /* G0_BANK_SWITCH_ENABLE */
				       0, /* G0_BANK_SWITCH_MODE */
				       NIB_G0_UP_DOWN |
				       NIB_G0_BANK_SWITCH_ENABLE |
				       NIB_G0_BANK_SWITCH_MODE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_INTERRUPT_A_ENABLE,
				       NIB_G0_GATE_INTERRUPT_ENABLE |
				       NIB_G0_TC_INTERRUPT_ENABLE,
				       NIB_G0_GATE_INTERRUPT_ENABLE |
				       NIB_G0_TC_INTERRUPT_ENABLE);
}

/* 
 * An IOCTL should supply us with 
 * Source input
 * Gate input
 * polarity 
 */
void
G0_Buffered_Event_Counting(struct ni_softc *sc) {
    uprintf("BEC!\n");
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_MODE, 0, NIB_G0_LOAD_SOURCE_SELECT);
    /* Here we setup the initial counter value -- ioctl? */
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_LOAD_A1, 1);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_LOAD_A2, 0);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_COMMAND, NIB_G0_LOAD);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_INPUT_SELECT, 
				       0 << 2| /* Source select */
				       0 | /* G0 Source Polarity */
				       1 << 7 | /* Gate select */
				       0 | /* G0_OR_GATE = 0 */
				       NIB_G0_OUTPUT_POLARITY |
				       0, /* GATE_SELECT_LOAD_SOURCE = 0 */
				       NIB_G0_SOURCE_SELECT |
				       NIB_G0_SOURCE_POLARITY |
				       NIB_G0_GATE_SELECT |
				       NIB_G0_OR_GATE | 
				       NIB_G0_OUTPUT_POLARITY |
				       NIB_G0_GATE_SELECT_LOAD_SOURCE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_MODE,
				       0 << 13 | /* G0_GATE_POLARITY */
				       1 << 8 | /* GO_OUTPUT_MODE */
				       NIB_G0_RELOAD_SOURCE_SWITCHING |
/* If buffered cumulative then G0_LOADING_ON_GATE = 0 */   
				       NIB_G0_LOADING_ON_GATE |
				       0 << 12 | /* G0_LOADING_ON_TC */
				       2 << 0 | /* G0_GATING_MODE */
				       0 << 2 | /* G0_GATE_ON_BOTH_EDGES */
				       3 << 3 | /* G0_TRIGGER_MODE_FOR_EDGE_GATE */
				       0 << 5 | /* G0_STOP_MODE */
				       0 << 10, /* G0_COUNTING_ONCE */
				       NIB_G0_GATE_POLARITY |
				       NIB_G0_OUTPUT_MODE | 
				       NIB_G0_RELOAD_SOURCE_SWITCHING |
				       NIB_G0_LOADING_ON_GATE |
				       NIB_G0_LOADING_ON_TC |
				       NIB_G0_GATING_MODE |
				       NIB_G0_GATE_ON_BOTH_EDGES |
				       NIB_G0_TRIGGER_MODE_FOR_EDGE_GATE |
				       NIB_G0_STOP_MODE |
				       NIB_G0_COUNTING_ONCE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_COMMAND,
				       1 << 5 | /* count up! */
				       0 | /* G0_BANK_SWITCH_ENABLE */
				       0, /* G0_BANK_SWITCH_MODE */
				       NIB_G0_UP_DOWN |
				       NIB_G0_BANK_SWITCH_ENABLE |
				       NIB_G0_BANK_SWITCH_MODE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_INTERRUPT_A_ENABLE,
				       NIB_G0_GATE_INTERRUPT_ENABLE,
				       NIB_G0_GATE_INTERRUPT_ENABLE |
				       NIB_G0_TC_INTERRUPT_ENABLE);
}

void
G0_Single_Pulse_Generation(struct ni_softc *sc) {
    uprintf("SPG!\n");
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_MODE, 0, NIB_G0_LOAD_SOURCE_SELECT);
    /* If single pulse generation */
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_LOAD_A1, 10);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_LOAD_A2, 0);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_COMMAND, NIB_G0_LOAD);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_LOAD_B1, 1000);
    DAQ_STC_Windowed_Mode_Write(sc, NI_G0_LOAD_B2, 0);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_MODE, NIB_G0_LOAD_SOURCE_SELECT,
				       NIB_G0_LOAD_SOURCE_SELECT);

    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_INPUT_SELECT, 
				       0 << 2| /* Source select */
				       0 | /* G0 Source Polarity */
				       31 << 7 | /* Gate select */
				       0 | /* G0_OR_GATE = 0 */
				       NIB_G0_OUTPUT_POLARITY |
				       0, /* GATE_SELECT_LOAD_SOURCE = 0 */
				       NIB_G0_SOURCE_SELECT |
				       NIB_G0_SOURCE_POLARITY |
				       NIB_G0_GATE_SELECT |
				       NIB_G0_OR_GATE | 
				       NIB_G0_OUTPUT_POLARITY |
				       NIB_G0_GATE_SELECT_LOAD_SOURCE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_MODE,
				       0 << 13 | /* G0_GATE_POLARITY */
				       1 << 8 | /* GO_OUTPUT_MODE */
				       NIB_G0_RELOAD_SOURCE_SWITCHING |
/* If buffered cumulative then G0_LOADING_ON_GATE = 0 */   
				       0 | 
/*				       NIB_G0_LOADING_ON_GATE | */
				       NIB_G0_LOADING_ON_TC | /*0 << 12 | G0_LOADING_ON_TC */
				       0 << 0 | /* G0_GATING_MODE */
				       0 << 2 | /* G0_GATE_ON_BOTH_EDGES */
				       2 << 3 | /* G0_TRIGGER_MODE_FOR_EDGE_GATE */
				       2 << 5 | /* G0_STOP_MODE */
				       0 << 10, /* G0_COUNTING_ONCE */
				       NIB_G0_GATE_POLARITY |
				       NIB_G0_OUTPUT_MODE | 
				       NIB_G0_RELOAD_SOURCE_SWITCHING |
				       NIB_G0_LOADING_ON_GATE |
				       NIB_G0_LOADING_ON_TC |
				       NIB_G0_GATING_MODE |
				       NIB_G0_GATE_ON_BOTH_EDGES |
				       NIB_G0_TRIGGER_MODE_FOR_EDGE_GATE |
				       NIB_G0_STOP_MODE |
				       NIB_G0_COUNTING_ONCE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_G0_COMMAND,
				       0 << 5 | /* count up! */
				       0 | /* G0_BANK_SWITCH_ENABLE */
				       0, /* G0_BANK_SWITCH_MODE */
				       NIB_G0_UP_DOWN |
				       NIB_G0_BANK_SWITCH_ENABLE |
				       NIB_G0_BANK_SWITCH_MODE);
    DAQ_STC_Windowed_Mode_Masked_Write(sc, NI_INTERRUPT_A_ENABLE,
				       0,
				       NIB_G0_GATE_INTERRUPT_ENABLE |
				       NIB_G0_TC_INTERRUPT_ENABLE);
}
