/*
 * Copyright (c) 2001 Murray Stokely, BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Driver for National Instruments Data Aquisition cards.
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, January 1999 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *
 * Murray Stokely
 * 24 Feb 2001
 */

#include <sys/ddi.h>
#include <sys/sunddi.h>
#include "ni_solaris.h"
#include "nireg.h"

/*
 * [1] Section 7.6.1 : Windowed Mode Register Access Example
 */

void 
DAQ_STC_Windowed_Mode_Write1(ni_softc *sc, uint16_t address, uint8_t value) {
/* XXX - Forte C 6 update1 on Solaris has a problem with this, we must
   be extra careful with the parenthesis.
    ddi_put16(sc->handle1, (uint16_t *) sc->mem1 + (NI_WINDOW_ADDRESS * 2), address);
*/
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_ADDRESS * 2)), address);
    ddi_put8(sc->handle1, (uint8_t *) (sc->mem1 + (NI_WINDOW_DATA * 2)), value);
/*
    bus_space_write_2(sc->bar1_bt, sc->bar1_bh, NI_WINDOW_ADDRESS * 2, address);
    bus_space_write_1(sc->bar1_bt, sc->bar1_bh, NI_WINDOW_DATA * 2, value);
*/
    sc->reg[address] = value;
}

void 
DAQ_STC_Windowed_Mode_Write(ni_softc *sc, uint16_t address, uint16_t value) {
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_ADDRESS * 2)), address);
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_DATA * 2)), value);
    sc->reg[address] = value;
}

void
DAQ_STC_Windowed_Mode_Masked_Write(ni_softc *sc, uint16_t address, uint16_t value, uint16_t mask) {
    uint16_t copy;
    copy = (sc->reg[address] & ~mask) | (value & mask);
    sc->reg[address] = copy;
/* Masked write is used with 0000 every time!!! */
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_ADDRESS * 2)), address);
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_DATA * 2)), copy);
}  

void 
DAQ_STC_Windowed_Mode_Strobe_Write(ni_softc *sc, uint16_t address, uint16_t value) {
    uint8_t copy;
    copy = sc->reg[address] | value;
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_ADDRESS * 2)), address);
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_DATA * 2)), copy);
}

uint8_t
DAQ_STC_Windowed_Mode_Read1(ni_softc *sc, uint16_t address) {
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_ADDRESS * 2)), address);
    return ddi_get8(sc->handle1, (uint8_t *) (sc->mem1 + (NI_WINDOW_DATA * 2)));
}

uint16_t
DAQ_STC_Windowed_Mode_Read(ni_softc *sc, uint16_t address) {
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_ADDRESS * 2)), address);
    return ddi_get16(sc->handle1, (uint16_t *) (sc->mem1 + (NI_WINDOW_DATA * 2)));
}


void
board_write(ni_softc *sc, uint16_t address, uint16_t value) {
    ddi_put16(sc->handle1, (uint16_t *) (sc->mem1 + address), value);
}

void
board_write1(ni_softc *sc, uint16_t address, uint8_t value) {
    ddi_put8(sc->handle1, (uint8_t *) (sc->mem1 + address), value);
}

uint16_t
board_read(ni_softc *sc, uint16_t address) {
    return ddi_get16(sc->handle1, (uint16_t *) (sc->mem1 + address));
}

uint16_t
board_read1(ni_softc *sc, uint8_t address) {
    return ddi_get8(sc->handle1, (uint8_t *) (sc->mem1 + address));
}
