/*
 * Copyright (c) 2001 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Driver for National Instruments PCI E-Series data acquisition cards.
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, May 1995 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *  [4] "MITE DMA Information", Tim Hayles October 19, 2000
 *  [5] MITE DMA Register Map (NI Confidential)
 *
 */

/* miniMITE DMA Registers / Defines */

/* From [5].  This could also be obtained from PCI-Eser.zip which is
   the source code to a DOS driver released on the NI ftp site. */

/* These are all offset from DMA_BASE which is (channel*0x100) + 0x500 */
#define DMA_BASE_ADDR(dmaChan) ((dmaChan * 0x0100L) + 0x500)

#define DMA_CHANNEL_OPERATION          0x00
#define DMA_CHANNEL_CONTROL            0x04
#define DMA_TRANSFER_COUNT             0x08
#define DMA_MEMORY_CONFIGURATION       0x0C
#define DMA_MEMORY_ADDRESS             0x10
#define DMA_DEVICE_CONFIGURATION       0x14
#define DMA_DEVICE_ADDRESS             0x18
#define DMA_LINK_CONFIGURATION         0x1C
#define DMA_LINK_ADDRESS               0x20
#define DMA_CHANNEL_STATUS             0x3C
#define DMA_FIFO_COUNT                 0x40

/* CHANNEL OPERATION (CHOR) BITS */
#define CHOR_DMA_RESET       0x80000000
#define CHOR_CLR_DONE        0x00000080
#define CHOR_FRESET          0x00000010
#define CHOR_ABORT           0x00000008
#define CHOR_START           0x00000001

#define DMA_LINK_SHORT_MODE  4
#define DMA_LINK_LONG_MODE   5

/* CHANNEL CONTROL (CHCR) BITS */
#define CHCR_SET_DMA_IE      0x80000000
#define CHCR_CLR_DMA_IE      0x40000000
#define CHCR_SET_DONE_IE     0x02000000
#define CHCR_CLR_DONE_IE     0x01000000
#define CHCR_DIR_INPUT       0x00000008
#define CHCR_DIR_OUTPUT      0x00000000
#define CHCR_BITS            0x000000F0 /* We must set this */

/* MEMORY CONFIGURATION REGISTER */
/* Always write 00E00600 */

/* DEVICE CONFIGURATION REGISTER */
/* Specifies request source, DMA channel, and some random bits */
/* Write 0x00E40241 for DMA channel 0 */

/* LINK CONFIGURATION REGISTER */
/* This probably sets the size of the pointer and stuff so look here it 
   this doesn't work on a non 32bit system. */
/* Always write 0x00E00700 */
