/*
 * Copyright (c) 2001 Murray Stokely
 * Copyright (c) 2001 BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Driver for National Instruments PCI E-Series data aquisition cards.
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, January 1999 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *
 * Murray Stokely
 * 24 Feb 2001
 */

#define NI_VENDOR_ID   0x1093
#define DEVICE_ID_6023 0x2A60
#define DEVICE_ID_6024 0x2A70
#define DEVICE_ID_6025 0x2A80
#define DEV_AI    0
#define DEV_DIO   1
#define DEV_GPCT0 2
#define DEV_GPCT1 3
#define DEV_STATS 4
#define NI_AIOPEN      0x0001
#define NI_DIOOPEN     0x0002
#define MAX_SIZE 256

#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#define VERBOSE(msg) cmn_err(CE_NOTE, msg)

/*
 * The ADC FIFO size is 512 (From [3] A-14) but there's no reason why we can't
 * keep more than that in memory.
 */

#define ADC_FIFO_SIZE 2048

typedef struct ni_struct {               /* per-unit structure */
    dev_info_t              *dip;
    kmutex_t                mutex;
    kcondvar_t              cv;
    caddr_t                 mem1;
    caddr_t                 mem2;
    off_t                   mem_size1;
    off_t                   mem_size2;
    ddi_acc_handle_t        handle1;
    ddi_acc_handle_t        handle2;
    int                     len;           /* length of text msg */
    int                     ni_state;    /* driver state */
    ddi_iblock_cookie_t     iblock_cookie; /* for mutexes */
    /* Analog Input variables */
    int                     scan_count;		/* 24bit scan counter  */
    int                     scan_interval;	/* 24bit scan interval counter */
    int                     sample_interval;	/* 16bit sample interval counter */
    int                     chancnt;
    uint32_t                channels[16];
    uint16_t                fifocnt;
    uint16_t                startcnt;
    /* Digital I/O variables */
    uint8_t dio;
    int wrmask;
    /* Soft copy of registers */
    int reg[256];
} ni_softc;

/* I/O Prototypes */

void DAQ_STC_Windowed_Mode_Write1(ni_softc *sc, uint16_t address, uint8_t value);
void DAQ_STC_Windowed_Mode_Write(ni_softc *sc, uint16_t address, uint16_t value);
void DAQ_STC_Windowed_Mode_Masked_Write(ni_softc *sc, uint16_t address, uint16_t value, uint16_t mask);
void DAQ_STC_Windowed_Mode_Strobe_Write(ni_softc *sc, uint16_t address, uint16_t value);
uint8_t DAQ_STC_Windowed_Mode_Read1(ni_softc *sc, uint16_t address);
uint16_t DAQ_STC_Windowed_Mode_Read(ni_softc *sc, uint16_t address);
void board_write(ni_softc *sc, uint16_t address, uint16_t value);
void board_write1(ni_softc *sc, uint16_t address, uint8_t value);
uint16_t board_read(ni_softc *sc, uint16_t address);
uint16_t board_read1(ni_softc *sc, uint8_t address);
