/*
 * Copyright (c) 2001 Murray Stokely
 * Copyright (c) 2001 BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Driver for National Instruments PCI E-Series data acquisition cards.
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, January 1999 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *
 */

struct ai_channel {
    enum {DIFF, RSE, NRSE} mode;
    uint8_t polarity;
    uint8_t gain;
    uint8_t num;
    uint8_t last;
};

struct gpct_event {
    enum {SIMPLE_EVENT_COUNTING, BUFFERED_EVENT_COUNTING} type;
    uint8_t source;
    uint8_t polarity;
    uint8_t gate;
    uint8_t output_polarity;
    uint8_t gate_polarity;
    uint8_t output_mode;
    uint8_t up_down;
};

struct ai_scaninfo {
    uint8_t scans;
    uint32_t scan_interval;
    uint16_t sample_interval;
};

struct daq_device {
    uint16_t device_id;
    char description[80];
    uint8_t ai_resolution;
    uint8_t ai_channels;
    uint8_t ao_resolution;
    uint8_t ao_channels;
    uint8_t dio_lines;
};

#define NICTL_DEVICE_INFO          _IOR('N', 100, struct daq_device)

#define NICTL_DIO_CONFPORT         _IOW('N', 1, int)
#define NICTL_DIO_SELPORT          _IOW('N', 2, int)
#define NICTL_DIO_WRMASK           _IOW('N', 3, int)

#define NICTL_AI_SCAN_START        _IOW('N', 1, struct ai_scaninfo)
#define NICTL_AI_SCAN_SETUP        _IOW('N', 2, struct ai_scaninfo)
#define NICTL_AI_RESET             _IOW('N', 3, int)
#define NICTL_AI_ADDCH             _IOW('N', 4, int)
#define NICTL_AI_ONEPT_SETUP       _IOW('N', 5, int)
#define NICTL_AI_CONVERT           _IOW('N', 6, struct ai_channel)
#define NICTL_AI_SETSCAN           _IOW('N', 7, int)

#define NICTL_GPCT_RESET           _IOW('N', 1, int)
#define NICTL_GPCT_ARM             _IOW('N', 2, int)
#define NICTL_GPCT_SETUP           _IOW('N', 3, struct gpct_event)

/*MALLOC_DECLARE(M_NIFIFO);
*/

/* 
Analog Input can be either [3]
   DIFF - Uses two analog channel input lines.  On eline connects to the positive input of the board programmable gain instrumentation amplifier (PGIA), and the other connects to the negative input of the PGIA.
   RSE
   NRSE
 Polarity
 Gain
*/
